<?php
namespace Neuron\Person\Controller;

use Neuron\Person\Attribute\Type as TypeModel;
use Neuron\Application\Framework as NAF;
use Neuron\Generic\Result as Result;

class AttributeType extends \Neuron\Person\Controller{

	protected function getView($name, $variables = array()) {

		/* default layout & view prefix */
		$layout = 'neuron/person/layout/form';
		$prefix = 'neuron/person/attribute';

		/* libraries */
		if(! class_exists('Neuron\Application\Framework\Entity')) {
			$this->using(
				'bootstrap-4',
				null,
				array(
					'https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js',
					'https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.7/umd/popper.min.js',
					'https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css',
					'https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/js/bootstrap.min.js'
				)
			);
		}

		/* get layout n view prefix from config */
		$config = $this->getConfig();
		if (isset($config['person'])) {
			$config = $config['person'];
			$layout = isset($config['formlayout']) ? $config['formlayout'] : $layout;
			$prefix = isset($config['viewprefix']) ? $config['viewprefix'] : $prefix;
		}

		/* set layout and view */
		$libs['Contact'] = class_exists('Neuron\Contact\Controller\Contact');
		$libs['Organization'] = class_exists('Neuron\Organization\Controller\Organization');
		$libs['Payroll'] = class_exists('Neuron\Payroll\Controller\Payroll');
		$variables['libs'] = $libs;

		$this->layout($layout);
		$view = new \Zend\View\Model\ViewModel($variables);
		$view->setTemplate($prefix . '/' . strtolower($name));

        // $view->setVariables(array(
        //     'sessmgt' => $this->getSession(),
        // ));

		return $view;
	}

	public function indexAction(){
		$layout = $this->layout();
		$layout->setVariables(['current' => 'Manage Attribute']);
		$this->getSession();
		$this->using('font-awesome', 10, 'https://use.fontawesome.com/releases/v5.8.1/css/all.css');
		$this->using('datatable', 20, array(
			'https://cdn.datatables.net/1.10.19/js/jquery.dataTables.min.js',
			'https://cdn.datatables.net/1.10.19/js/dataTables.bootstrap4.min.js',
			'https://cdn.datatables.net/1.10.19/css/dataTables.bootstrap4.min.css'
		));
		$this->using('jquery.validate',21);
		$this->usingJs('bootbox',21,  array(
			'https://cdnjs.cloudflare.com/ajax/libs/bootbox.js/4.4.0/bootbox.min.js'
		));
		$this->using('generic',24);
		$this->usingJs('moment', 23, array(
			'https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.29.1/moment.min.js'
		));

		return $this->getView('attribute-type');
	}

	public function loadDatatableAction(){
		$post 				= $this->getRequest()->getPost();
		$typeModel 	= new TypeModel(TypeModel\Storage::factory($this->getDb(),$this->getConfig()));
		$params 			= array(
			'order' 		=> array(
				'columnName' 	=> $post['columns'][$post['order'][0]['column']]['data'],
				'dir' 			=> $post['order'][0]['dir']
			),
			'search' 		=> $post['search']['value'],
			'limit' 		=> $post['length'],
			'offset' 		=> $post['start']
		);

		$result 			= $typeModel->loadList(null,TypeModel\Storage::LIST_ALL,$params);

		if($result->code == 0){
			
			$result = $result->data;
			
			// if ($manager = NAF\Event\Manager::get()) {
				$config = $this->getConfig();
				if ($config['flag_source_ext'] == 'idmt') {
					$mappingAttribute = new \Neuron\Addon\Person\ListAttribute\MappingAttribute(\Neuron\Addon\Person\ListAttribute\MappingAttribute\Storage::factory($this->getDb(),$this->getConfig()));
					foreach ($result->data as $key => $value) {
						$attrType = '';
						if ($mappingAttribute->load($value['attr_code'], \Neuron\Addon\Person\ListAttribute\MappingAttribute\Storage::LOADBY_CODE)) {
							$attrType = $mappingAttribute->attr_type;
						}
						$result->data[$key]['attr_type'] = $attrType;
					}
				}
			// }
			
			$res 			= array(
				'draw' 				=> $post['draw'],
				'data' 				=> $result->data,
				'recordsFiltered' 	=> $result->countTotal,
				'recordsTotal'		=> $result->countTotal,
				"from" 				=> $from,
				'hash' 				=> $hash
			);
		}else{

		}
		return $this->getOutput(json_encode($res));
	}

	public function loadListAction(){
		$modelAttribute 		= new TypeModel(TypeModel\Storage::factory($this->getDb(),$this->getConfig()));
		$result 				= $modelAttribute->loadList();
		$rows 		= $result->data->list;
		$newRows 	= [];
		foreach($rows as $row){
			$newRows[] = $row;
		}
		$result->data->list = $newRows;
		
		$config = $this->getConfig();
		if(isset($config['flag_source_ext']) && $config['flag_source_ext'] == 'idmt'){
			if ($result->code == Result::CODE_SUCCESS) {
				foreach ($result->data->data as $key => $value) {
					$mappingAttribute = new \Neuron\Addon\Person\ListAttribute\MappingAttribute(\Neuron\Addon\Person\ListAttribute\MappingAttribute\Storage::factory($this->getDb(),$this->getConfig()));
					if ($mappingAttribute->load($value['attr_code'], \Neuron\Addon\Person\ListAttribute\MappingAttribute\Storage::LOADBY_CODE)) {
						$result->data->data[$key]['attr_type'] = $mappingAttribute->attr_type;
					} else {
						$mappingAttribute->attr_code = $value['attr_code'];
						$mappingAttribute->attr_type = 'TEXT';
						$mappingAttribute->save();
						$result->data->data[$key]['attr_type'] = 'TEXT';
					}
					
					if (strtoupper($value['attr_code']) == 'BANK') {
						
						$sendParam = array(
										'guid' => 0,
										'code' => 0,
										'data' => json_encode(array('code'=>'BANK')),
									);
						
						$params = new \Neuron\Application\Idmt\Params(\Neuron\Application\Idmt\Params\Storage::factory($this->getDb(),$this->getConfig()));
						$resultParams = $params->loadList($sendParam);
						
						if ($resultParams->code == Result::CODE_SUCCESS) {
							$result->data->data[$key]['attr_data'] = $resultParams->data;
						}
						
					}
					
					if (strtoupper($value['attr_code']) == 'REG') {
						
						$sendParam = array(
										'guid' => 0,
										'code' => 0,
										'data' => json_encode(array('code'=>'REG')),
									);
						
						$params = new \Neuron\Application\Idmt\Params(\Neuron\Application\Idmt\Params\Storage::factory($this->getDb(),$this->getConfig()));
						$resultParams = $params->loadListArea($sendParam);
						
						if ($resultParams->code == Result::CODE_SUCCESS) {
							$result->data->data[$key]['attr_data'] = $resultParams->data;
						}
						
					}
					
					if (strtoupper($value['attr_code']) == 'WITEL') {
						
						$sendParam = array(
										'guid' => 0,
										'code' => 0,
										'data' => json_encode(array('code'=>'WITEL')),
									);
						
						$params = new \Neuron\Application\Idmt\Params(\Neuron\Application\Idmt\Params\Storage::factory($this->getDb(),$this->getConfig()));
						$resultParams = $params->loadListArea($sendParam);
						
						if ($resultParams->code == Result::CODE_SUCCESS) {
							$result->data->data[$key]['attr_data'] = $resultParams->data;
						}
						
					}
					
					if (strtoupper($value['attr_code']) == 'STO') {
						
						$sendParam = array(
										'guid' => 0,
										'code' => 0,
										'data' => json_encode(array('code'=>'STO')),
									);
						
						$params = new \Neuron\Application\Idmt\Params(\Neuron\Application\Idmt\Params\Storage::factory($this->getDb(),$this->getConfig()));
						$resultParams = $params->loadListArea($sendParam);
						
						if ($resultParams->code == Result::CODE_SUCCESS) {
							$result->data->data[$key]['attr_data'] = $resultParams->data;
						}
						
					}
					
					if (strtoupper($value['attr_code']) == 'SECTOR') {
						
						$sendParam = array(
										'guid' => 0,
										'code' => 0,
										'data' => json_encode(array('code'=>'SECTOR')),
									);
						
						$params = new \Neuron\Application\Idmt\Params(\Neuron\Application\Idmt\Params\Storage::factory($this->getDb(),$this->getConfig()));
						$resultParams = $params->loadListArea($sendParam);
						
						if ($resultParams->code == Result::CODE_SUCCESS) {
							$result->data->data[$key]['attr_data'] = $resultParams->data;
						}
						
					}
					
					// print_r();die();
					// if (isset($config['listAttribute'][strtoupper($value['attr_code'])])) {
						
					// }
					
				}
			}
		}
		
		return $this->getREST()->getResponse($result);
	}

	public function saveAction(){
		$sessmgt 	= $this->getSession()->get();
		$post 		= $this->getRequest()->getPost();
		$typeModel 	= new TypeModel(TypeModel\Storage::factory($this->getDb(),$this->getConfig()));

		
		$result 	= $typeModel->save($post);

		// Create document type
		$generic 	= new \Neuron\Person\Generic;
		$cache 		= $generic->getCacheAdapter();
		$cacheHash 	= 'documenttypeperson';
		if($cache->hasItem($cacheHash)){
			$dataDocument 	= json_decode($cache->getItem($cacheHash),true);
		}else{
			$documentType 	= new \Neuron\Document\Type(
				\Neuron\Document\Type\Storage::factory(
					$this->getDb(),
					$this->getConfig()
				)
			);
			$dataDocument 			= $documentType->loadByName("Person");
			$cache->setItem($cacheHash,json_encode($data));
		}

		$documentParentTypeId 		= $dataDocument['document_type_id'];

		$documentType 	= new \Neuron\Document\Type(
			\Neuron\Document\Type\Storage::factory(
				$this->getDb(),
				$this->getConfig()
			)
		);
		$documentTypeData = array(
            "document_type_id"  =>  null, 
            "parent_type_id"    =>  $documentParentTypeId, 
            "name"              =>  $post['attr_name'], 
            "description"       =>  $post['attr_name']." Person", 
		);
		$documentType->push($documentTypeData);
		$documentType->save(false);



		$config = $this->getConfig();
		if ($config['flag_source_ext'] == 'idmt') {
			$mappingAttribute = new \Neuron\Addon\Person\ListAttribute\MappingAttribute(\Neuron\Addon\Person\ListAttribute\MappingAttribute\Storage::factory($this->getDb(),$this->getConfig()));
			if ($mappingAttribute->load($post['attr_code'], \Neuron\Addon\Person\ListAttribute\MappingAttribute\Storage::LOADBY_CODE)) {
				$mappingAttribute->attr_type = $post['attr_type'];
				$mappingAttribute->save(true);
			} else {
				$mappingAttribute->attr_code = $post['attr_code'];
				$mappingAttribute->attr_type = $post['attr_type'];
				$mappingAttribute->save();
			}
		}
		
		$result->info = $this->translate($result->info);

		$responseResult = array(
			'guid' => $result->guid,
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 
		
		$logparam['subject'] = 'SAVE PERSON ATTRIBUTE TYPE';
		$logparam['identity'] = $this->getSession()->owner();;
		$logparam['data1'] = $result->data;
		$logparam['description'] = 'Log proses save person attribute type';
		$logparam['request'] = json_encode($post);
		$logparam['response'] = json_encode($responseResult);
		
		$this->writeLogAction($logparam);

		$generic 	= new \Neuron\Person\Generic;
		$generic->clearCache(md5(TypeModel::CACHE_HASH.'.loadList'));
		
		return $this->getREST()->getResponse($result);
	}

	public function deleteAction(){
		$post 		= $this->getRequest()->getPost();
		$result 	= new \Neuron\Generic\Result;
		if(!$post['attr_type_id'] || !isset($post['attr_type_id'])){
			$result->info 	= 'Invalid type id!';
			$result->code   = 400;
			$result->info   = $this->translate($result->info);
			return $this->getREST()->getResponse($result);
		}

		$typeModel 	= new TypeModel(TypeModel\Storage::factory($this->getDb(),$this->getConfig()));
		$result 			= $typeModel->remove($post['attr_type_id']);
		$result->info   = $this->translate($result->info);
		
		$responseResult = array(
			'guid' => $result->guid,
			'code' => $result->code,
			'info' => $result->info,
			'data' => $result->data
		); 
		
		$logparam['subject'] = 'DELETE PERSON ATTRIBUTE TYPE';
		$logparam['identity'] = $this->getSession()->owner();;
		$logparam['data1'] = $result->data;
		$logparam['description'] = 'Log proses delete person attribute type';
		$logparam['request'] = json_encode($post);
		$logparam['response'] = json_encode($responseResult);
		
		$generic 			= new \Neuron\Person\Generic;
		$generic->clearCache(md5(TypeModel::CACHE_HASH.'.loadList'));
		
		return $this->getREST()->getResponse($result);
	}

	public function loadListAreaAction() {
		$post = $this->getRequest()->getPost();
		//ok change to branch 109
		if (isset($post['attr_code']) && !empty($post['attr_code'])) {
			$code = strtoupper($post['attr_code']);
			$value = strtoupper($post['attr_value']);

			if ($code == 'REG') {
				$paramsArea['code'] = 'WITEL';
				$paramsArea['regional'] = $value;
			} else if ($code == 'WITEL') {
				$paramsArea['code'] = 'STO';
				$paramsArea['witel'] = $value;
			} else if ($code == 'STO') {
				$paramsArea['code'] = 'SECTOR';
				$paramsArea['sto'] = $value;
			}

			$sendParam = array(
				'guid' => 0,
				'code' => 0,
				'data' => json_encode($paramsArea),
			);
			
			$params = new \Neuron\Application\Idmt\Params(\Neuron\Application\Idmt\Params\Storage::factory($this->getDb(),$this->getConfig()));
			$result = $params->loadListArea($sendParam);

			$result->info = @$paramsArea['code'];
		}

		return $this->getREST()->getResponse($result);
	}

}
