<?php

namespace Neuron\Person\Experience\Storage;

class Mysql extends \Neuron\Db\Storage implements Skeleton {
    
    protected $_nafManager;
    
	//fungsi lama
	public function listdata($params) {

		$select = $this->select()->from(array('exp' => $this->_tables['person_experiences']));

		if ((!empty($params['person_id']))) {
            $select->where(array('exp.person_id' => $params['person_id']));
        }
		
		if(isset($params['order'][0]['column']) && !empty($params['order'][0]['column'])){
			$dir = $params['order'][0]['dir'];
			$i = $params['order'][0]['column'];
			$select->order(array($params['columns'][$i]['data'] . " " . $dir));
		}
	
		if(isset($params['length']) && !empty($params['length'])){
			$select->limit(intval($params['length']));
			$select->offset(intval($params['start']));
		}
		
		// echo $params->length;

		// echo str_replace('"','',$select->getSqlString()).'<br />';

        $data = $this->fetchAll($select);
        $dataAll = [];

        if($data) {
            foreach($data as $row) {
                array_push($dataAll, $row);
            }
		}

		// PAGINATION
		$selectC = $this->select()->from(array('exp' => $this->_tables['person_experiences']));
        $selectC->columns(['total_count' => new \Zend\Db\Sql\Expression('COUNT(*)')]);
       	if ((!empty($params['person_id']))) {
            $selectC->where(array('exp.person_id' => $params['person_id']));
        }
        $dataCount = $this->fetchRow($selectC);

		$total = ($dataCount['total_count'] > 0) ? $dataCount['total_count']:0;
		// END PAGINATION
                        
        $result = ['data' => $dataAll, 'total' => $total];
        
        return $result;
	}

	public function load($perExpId) {
		$select = $this->select()
						->from($this->_tables['person_experiences'])
						->where(array('experience_id' => $perExpId));

		return $this->fetchRow($select);
	}
	
	public function save(\Neuron\Person\Experience $perExp, $update = false) {
		if ($update) {
			
			$update = $this->update()
				->table($this->_tables['person_experiences'])
				->set($this->__($perExp->pull()))
				->where($this->__(array('experience_id' => $perExp->experience_id)));

			// echo str_replace('"','',$update->getSqlString()).'<br />';
			if ($this->execute($update)) {
					return $perExp->experience_id;
			}
			
		} else {

			$insert = $this->insert()
				->into($this->_tables['person_experiences'])
                ->autoincrement($this->_('experience_id'))
				->values($this->__($perExp->pull()));
			$save = $this->execute($insert);
			if ($save) {
					$perExp->experience_id = $save;
					return $perExp->experience_id;
			}
		}

		return false;
	}

	public function remove($perExpId) {
		$delete = $this->delete()
						->from($this->_tables['person_experiences'])
						->where(array('experience_id' => $perExpId));
        
		return $this->execute($delete);
	}

	public function loadList($ids, $mode = \Neuron\Person\Experience\Storage::LIST_ALL, $params = null) {
        /* base select */
        $select = $this->select()
                ->from(array('a' => $this->_tables['person_experiences']));

        /* loadby page */
        if ($mode == \Neuron\Person\Experience\Storage::LISTBY_PAGE) {
			if(isset($params['person_id']) && !empty($params['person_id'])){
				$select->where(array($this->_('person_id', 'a.') => $params['person_id']));
			}

            /* order */
            if(array_key_exists('order', $params) && array_key_exists('columns', $params)){
                foreach($params['order'] as $order){
                    $columnName = $params['columns'][$order['column']]['data'];
                    if($columnName == 'user_status_name'){
                        $colIdentity = $this->_('name', 'b.');
                    } else {
                        $colIdentity = $this->_($columnName, 'a.');
                    }
                    $select->order($colIdentity . ' ' . $order['dir']);
                }
            }
            
            /* get pager */
            $pager = \Neuron\Db\Storage\Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);

            /* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('company_name', 'd.'), '%' . trim($ids) . '%')
                    ->or
                    ->like($this->_('position', 'd.'), '%' . trim($ids) . '%')
					->or
                    ->like($this->_('job_desc', 'd.'), '%' . trim($ids) . '%')
                ->unnest();
            }

        } else {

            $pager = new \Neuron\Db\Storage\Pager(1, 999999);

            switch ($mode) {
                case \Neuron\Person\Experience\Storage::LISTBY_PERSON_TYPE:
                    $select->where(array($this->_('person_id', 'a.') => $ids));
                    break;
                case \Neuron\Person\Experience\Storage::LISTBY_TYPE:
                    $select->where(array($this->_('education_type', 'a.') => $ids));
                    break;
                default:
            }

        }

        $list = is_array($ids) ? $ids : ['id' => $ids];
        $list = is_array($params) ? array_merge($list, $params) : $list;
        $this->triggerQueryPerson($select, $list, __FUNCTION__);

        // echo str_replace('"','',$select->getSqlString()); die();
        try {
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    private function triggerQueryPerson($query, $params, $caller) {

        /* trigger before query */
        if ($this->_nafManager = \Neuron\Application\Framework\Event\Manager::get()) {
            $this->_nafManager->events()->trigger('person_experiences_query', $query, $params, $caller);
        }
    }

}