<?php

namespace Neuron\Person\Parameter\Storage;

class Mysql extends \Neuron\Db\Storage implements Skeleton {
    protected $_nafManager = null;

    public function getTotalRows() {
		try {

			$select = $this->select()
						->columns(array('total' => $this->expression('FOUND_ROWS()')));
														 
			return $this->fetchValue($select);  

		} catch (\Exception $ex) {
			return false;
		}              
	} 

	public function listparam($ids = null, $params = array()) {
		try {
			$select = $this->select();
			$select ->from($this->_tables['person_parameters']);
			$select ->where(array('(end_date is null or end_date > now())'));

			if (!empty($ids)) {
				$select ->where(array('param_ref_name'	=> $ids));
			}

			if (!empty($params['search']['value'])) {
				$select->where(array('(value1 LIKE "%'.$params['search']['value'].'%")'));
			}
			
			$select ->order(array('param_ref_name', 'param_ref_id'));         
			// echo str_replace('"','',$select->getSqlString()); die();     
			return $this->fetchAll($select);
		} catch (\Throwable $th) {
			return $th->getMessage();
		}
	}

	public function lists($page = 0, $rowCount = 10, $filter = array(), $order  = array())
	{
		try {

			$select = $this->select()
						->quantifier($this->expression('SQL_CALC_FOUND_ROWS'))
						->from(array('pcc' => $this->_tables['person_parameters']));
            if($rowCount > 0 ){
				$select->limit(intval($rowCount))
				        ->offset(intval($page));
            }
            
			if(isset($filter['searchValue'])){
				if($filter['searchValue'] != '' || $filter['searchValue'] != null){
					$select->where(array('(param_ref_name like "%'.$filter['searchValue'].'%" OR description like "%'.$filter['searchValue'].'%" OR value1 like "%'.$filter['searchValue'].'%")'));
				}
			}

			if(isset($order['by'])){
				if(@$order['by'] != '' && @$order['by'] != null){
					$select->order($order['by'].' '.$order['value']);
				}
			}

			$select->where(array('(end_date is null or DATE(end_date) >= now())'));
			
			// echo str_replace('','',$select->getSqlString()).'<br />'; die();         
			return $this->fetchAll($select);  

		} catch (\Exception $ex) {
			return false;
		}  
	}

	public function load($paramId) {
		$select = $this->select()
						->from($this->_tables['person_parameters'])
						->where(array('param_id' => $paramId));

		return $this->fetchRow($select);
	}
	
	public function save(\Neuron\Person\Parameters $param, $update = false) {
		if ($update) {
			
			$update = $this->update()
				->table($this->_tables['person_parameters'])
				->set($this->__($param->pull()))
				->where($this->__(array('param_id' => $param->param_id)));

			// echo str_replace('"','',$update->getSqlString()).'<br />';
			if ($this->execute($update)) {
					return $param->param_id;
			}
			
		} else {

			$insert = $this->insert()
							->into($this->_tables['person_parameters'])
							->values($this->__($param->pull()));
								// echo str_replace('"','', $insert->getSqlString())."<br />"; exit;
			$save = $this->execute($insert);
			if ($save) {
					$param->param_id = $save;
					return $param->param_id;
			}
		}

		return false;
	}

	public function remove($data = array()) {
		try {
			$update = $this->update()
							->table($this->_tables['person_parameters'])	
							->set(array('end_date'=> $data['end_date']))
							->where(array('param_id'=>$data['param_id']));
			// echo str_replace('"','', $update->getSqlString())."<br />"; exit;
			if($this->execute($update)){
				return $data['param_id'];
			}
			return false;
		} catch (Exception $e) {
			return false;
		}
	}

	public function loadBY($filter= array()) {
		$select = $this->select()
						->from($this->_tables['person_parameters'])
						->where(array('(end_date is null or end_date > now())'));
		if($filter){
			$select->where($filter);
		}

		return $this->fetchRow($select);
	}

	public function deleteFromDb($id){
		try {
			$delete = $this->delete()
							->from($this->_tables['person_parameters'])
							->where(array('param_id'=>$id));
			return $this->execute($delete);
		} catch (Exception $e) {
			return false;
		}
	}

}