<?php
namespace Neuron\Person\v2;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;

class Attribute extends \Neuron\Generic\Entity\Mapper {

    protected $_storage;
    const PULL_ALL = 3;

    public function __construct(Attribute\Storage\Skeleton $storage,$data = array()) {

        /* simpan storage */
        $this->_storage = $storage;

        /* map properti ke tabel */
        parent::__construct(
            $data,
            array(
                new Map('person_attribute_id'),
                new Map('attr_code'),
                new Map('person_id'),
                new Map('attr_value'),
                new Map('start_date'),
                new Map('end_date'),
                new Map('document_id'),
                new Map('update_dtm'),
                new Map('update_by'),
                new Map('tmp_file',null,true)
            ),
            parent::CASE_INSENSITIVE
        );
    }

    public function load($id, $mode = Attribute\Storage::LOADBY_ID,$params = array()) {
        /* query */
        if ($data = $this->_storage->load($id, $mode, $params)) {
            
            $this->push($data);

            /* berhasil */
            return $this->person_attribute_id;
        }

        /* default gagal */
        return false;
    }

    public function save($update = false) {

        /* simpan */
        if ($id = $this->_storage->save($this, $update)) {

            /* set id */
            $this->id = $id;
            return $id;
        }

        /* return hasil */
        return false;
    }


    public function saveAttributeFile($file,$attributeOwner,$attr_code,$sessOwner){
        $result     = new Result;
		// Simpan attribute files to document
		$uploadConfig 	= $this->_storage->getConfig()['upload_document'];
		$uploadPath = $uploadConfig['location'];
		if($uploadConfig['split_folder']){
			$uploadPath = $uploadPath . '/' . date($uploadConfig['split_folder']);
		}

		$upload 		= new \Neuron\Upload\Upload(
			'uploader', $this->_storage->getConfig()['upload_document']
		);

        $upload->handle(array('file' => $file));
        $file_name 	        = $upload->name;
        $file_size 	        = $upload->size;
        $file_type 	        = pathinfo($upload->name, PATHINFO_EXTENSION);
        $file_dir 	        = $uploadPath;

        $documentType 		= new \Neuron\Document\Type(
            \Neuron\Document\Type\Storage::factory(
                $this->_storage->getAdapter(),
                $this->_storage->getConfig()
            )
        );
        $dataDocumentType 		= $documentType->loadByName($attr_code);
        $documentTypeId 		= $dataDocumentType['document_type_id'];

        $document 			= new \Neuron\Document\Entity(
            \Neuron\Document\Entity\Storage::factory(
                $this->_storage->getAdapter(),
                $this->_storage->getConfig()
            )
        );
        
        $dataDocument 		= array(
            "document_no"		=>	"EMP/ATTR/".$attr_code."/".$attributeOwner.'/'.date('Ymd'),
            "document_name"		=>	"Employee Attribute File (".$attr_code.")",
            "document_desc"		=>	"Employee Attribute File (".$attr_code.")",
            "document_dtm"		=>	date('Y-m-d H:i:s'),
            "document_type_id"	=>	$documentTypeId,
            "document_by"		=>	$sessOwner,
            "document_end_dtm"	=> 	null,
        );

        $document->push($dataDocument);
        if($documentId  = $document->save(false)){
            $documentFiles 	= new \Neuron\Document\Files(
                \Neuron\Document\Files\Storage::factory(
                    $this->_storage->getAdapter(),
                    $this->_storage->getConfig()
                )
            );
            $dataFiles 		= array(
                "document_id" => $document->document_id,
                "file_name" => $file_name,
                "file_dir" => $file_dir,
                "file_type" => $file_type,
                "file_size" => $file_size,
                "document_file_status" => 1,
            );

            $documentFiles->push($dataFiles);
            $saveFile = $documentFiles->save(false);
            if($saveFile){
                $documentOwner 	= new \Neuron\Document\Owner(
                    \Neuron\Document\Owner\Storage::factory(
                        $this->_storage->getAdapter(),
                        $this->_storage->getConfig()
                    )
                );
                $ownerData 		= array(
                    "document_id"	 	=> $document->document_id,
                    "owner_ref_code" 	=> "EMPLOYEE",
                    "owner_ref_id" 		=> $attributeOwner,
                    "domain_name" 		=> null,
                    "domain_id" 		=> null,
                );
                $documentOwner->push($ownerData);
                if($documentOwner->save(false)){	
                    $audit 	= new \Neuron\Document\Audit(
                        \Neuron\Document\Audit\Storage::factory(
                            $this->_storage->getAdapter(),
                            $this->_storage->getConfig()
                        )
                    );
                    $dataAudit 	= array(
                        "document_id" => $document->document_id,
                        "user_id" => $sessOwner,
                        "audit_type_id" => 1,
                        "audit_dtm" => date('Y-m-d'),
                        "notes" => null
                    );
                }
            }
        }

		$result->data 	= $documentId;
		return $result;
    }

    public function saveEx($data){
        $this->clear();
        $result  = new Result;
        $this->load(null,Attribute\Storage::LOADBY_ATTR_CODE_PERSON_ID,$data);

        // Load dulu, datanya udah ada atau belum;
        if($this->person_attribute_id){

            if(!$data['attr_value']){
                // Kalo kosong attribute valuenya, di end_date aja
                $this->end_date = $this->_storage->now();
                $this->save(true);
            }else{
                $changed   = false;

                
                if($data['tmp_file']){
                    $changed   = true;
                    $saveDocument   = $this->saveAttributeFile(
                        $data['tmp_file'],
                        $data['person_id'],
                        $data['attr_code'],
                        $data['update_by']
                    );
                    if($saveDocument->code == 0){
                        $documentId         = $saveDocument->data;
                        $this->document_id  = $documentId;
                    }
                }

                if($this->attr_value != $data['attr_value']){
                    $changed            = true;
                    $this->attr_value   = $data['attr_value'];
                }
                if($changed){
                    $this->update_dtm   = $this->_storage->now();

                    if($this->save(true)){
                        $result->data   = $this->pull();
                    }else{
                        $result->info   = "Failed saving attribute";
                        $result->data   = $this->pull();
                    }
                }
            }

        }else{
            $this->push($data);

            // Create 
            if(!$this->attr_value){
                $result->code = 0;
                $result->info = "Empty value on create. Skipped.";
                $result->data = $this->pull();
                return $result;
            }

            if($this->tmp_file){
                // Upload filenya dulu
                $uploadFile = $this->saveAttributeFile(
                    $this->tmp_file,
                    $this->person_id,
                    $this->attr_code,
                    $this->update_by
                );
                if($uploadFile->code == 0){
                    $this->document_id = $uploadFile->data;
                }
            }
            if($this->save()){
                $result->data = $this->pull();
            }else{
                $result->code = 1;
                $result->info = "Failed saving attribute";
                $result->data = $this->pull();
            }
        }
        
        return $result;
    }
    
    public function pull($mode = parent::PULL_NOTSKIP, $data = NULL) {
        $list = array();
        $item = parent::pull();
        if($mode == self::PULL_ALL) {
            $item['tmp_file'] = $this->tmp_file;
        }

        $list = $item;
        return $list;
    }
}
